__version__ = "0.2.24"


# from ._services.aiplatform.live_predict import get_image_classification_prediction_from_deployed_automl

# __all__ = ["get_image_classification_prediction_from_deployed_automl"]